unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons, ComCtrls, atMath, StdCtrls, Menus,
  fSimplifiedModel, f_info;

type
  TMathModel = class;

  TForm1 = class(TForm)
    atMathEvaluator1: TatMathEvaluator;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    New1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    Saveas1: TMenuItem;
    Quit1: TMenuItem;
    N1: TMenuItem;
    Show1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    Simplifiedmodel1: TMenuItem;
    Panel1: TPanel;
    Panel5: TPanel;
    Panel6: TPanel;
    PaintBox1: TPaintBox;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Panel7: TPanel;
    Label4: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    edXStart: TEdit;
    Label8: TLabel;
    edXEnd: TEdit;
    Label9: TLabel;
    edYStart: TEdit;
    Label10: TLabel;
    edYEnd: TEdit;
    Label11: TLabel;
    Label12: TLabel;
    edCounterStart: TEdit;
    Label13: TLabel;
    edCounterEnd: TEdit;
    Label14: TLabel;
    edCounterResolution: TEdit;
    Bevel1: TBevel;
    btGraphic: TBitBtn;
    Splitter1: TSplitter;
    Panel2: TPanel;
    lbMathModel: TLabel;
    Plotrefresh1: TMenuItem;
    Panel3: TPanel;
    cbSyntax: TCheckBox;
    atEquationMemo1: TatEquationMemo;
    procedure FormCreate(Sender: TObject);
    procedure btGraphicClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Simplifiedmodel1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Saveas1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure Quit1Click(Sender: TObject);
    procedure cbSyntaxClick(Sender: TObject);
  private
    { Private declarations }
    FMathModel : TMathModel;
    procedure UpdateMathModel;
    procedure UpdateScreen;
    procedure LoadModelFromFile( AFileName:string );
    procedure SaveModelToFile( AFileName:string );
    procedure PaintGraphic;
  public
    { Public declarations }
  end;

  TMathModel = class(TComponent)
  private
    { Properties }
    FCounterStart      : extended;
    FCounterEnd        : extended;
    FCounterResolution : integer;
    FXAxisStart        : extended;
    FXAxisEnd          : extended;
    FYAxisStart        : extended;
    FYAxisEnd          : extended;
    FModel             : TStrings;
    { auxiliary variables }
    FCounterVar        : extended;
    FFileName          : string;
    procedure SetFileName( value:string );
  public
    property FileName:string read FFileName write SetFileName;
  published
    property CounterStart: extended read FCounterStart write FCounterStart;
    property CounterEnd: extended read FCounterEnd write FCounterEnd;
    property CounterResolution: integer read FCounterResolution  write FCounterResolution;
    property XAxisStart: extended read FXAxisStart write FXAxisStart;
    property XAxisEnd: extended read FXAxisEnd write FXAxisEnd;
    property YAxisStart: extended read FYAxisStart write FYAxisStart;
    property YAxisEnd: extended read FYAxisEnd write FYAxisEnd;
    property Model:TStrings read FModel write FModel;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TMathModel }

procedure TMathModel.SetFileName( value:string );
begin
   FFileName:=value;
   if FFileName>'' then
      Form1.lbMathModel.Caption:='Math model - '+FFileName
   else
      Form1.lbMathModel.Caption:='Math model - (new)';
   Form1.atEquationMemo1.SetFocus;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
   { Creates variables into atMathEvaluator }
   FMathModel:=TMathModel.Create(self);
   FMathModel.FModel:=atEquationMemo1.Equations;
   with atMathEvaluator1.Variables, FMathModel do
   begin
      Add('C',       @FCounterVar,nil);
      Add('C_start', @FCounterStart, nil);
      Add('C_end',   @FCounterEnd, nil);
      Add('C_resol', @FCounterResolution, nil);
      Add('X_start', @FXAxisStart, nil);
      Add('X_end',   @FXAxisEnd, nil);
      Add('Y_start', @FYAxisStart, nil);
      Add('Y_end',   @FYAxisEnd, nil);
   end;
end;

procedure TForm1.UpdateMathModel;
begin
   with FMathModel do
   begin
      { update property values }
      CounterStart      := StrToFloat(edCounterStart.Text);
      CounterEnd        := StrToFloat(edCounterEnd.Text);
      CounterResolution := StrToInt(edCounterResolution.Text);
      XAxisStart        := StrToFloat(edXStart.Text);
      XAxisEnd          := StrToFloat(edXEnd.Text);
      YAxisStart        := StrToFloat(edYStart.Text);
      YAxisEnd          := StrToFloat(edYEnd.Text);
   end;
end;

procedure TForm1.UpdateScreen;
begin
   with FMathModel do
   begin
      { update property values }
      edCounterStart.Text      := FloatToStr(CounterStart);
      edCounterEnd.Text        := FloatToStr(CounterEnd);
      edCounterResolution.Text := IntToStr(CounterResolution);
      edXStart.Text            := FloatToStr(XAxisStart);
      edXEnd.Text              := FloatToStr(XAxisEnd);
      edYStart.Text            := FloatToStr(YAxisStart);
      edYEnd.Text              := FloatToStr(YAxisEnd);
   end;
end;

procedure TForm1.PaintGraphic;
var C      : integer;
    X,Y    : extended;
    Cx,Cy  : integer;
    Step   : extended;
    XScale : extended;
    YScale : extended;
    XIndex : integer;
    YIndex : integer;
    RIndex : integer;
    GIndex : integer;
    BIndex : integer;
    cl     : integer;
begin
   with FMathModel, PaintBox1, Canvas, atMathEvaluator1 do
      if Equations.Count>0 then
      begin
         { prepare the canvas }
         FillRect(ClientRect);
         Pen.Color:=clNavy;
         { Equation compiling }
         for c:=0 to Equations.Count-1 do
            if not Equations[c].Check then
               raise Exception.Create('Equations are not valid');
         Xindex:=Equations.FunctionIndexByName('X');
         Yindex:=Equations.FunctionIndexByName('Y');
         RIndex:=Equations.FunctionIndexByName('Red');
         GIndex:=Equations.FunctionIndexByName('Green');
         BIndex:=Equations.FunctionIndexByName('Blue');
         if Xindex=-1 then
            raise Exception.Create('"X" function not defined');
         if Yindex=-1 then
            raise Exception.Create('"Y" function not defined');
         if Rindex+Bindex+Gindex=-3 then
            raise Exception.Create('"RED","GREEN" and "BLUE" not defined');
         { prepare the independent varible to start }
         Step:=(FCounterEnd-FCounterStart)/(FCounterResolution-1);
         FCounterVar:=FCounterStart;
         XScale:=(ClientWidth-1)/(FXAxisEnd-FXAxisStart);
         YScale:=(ClientHeight-1)/(FYAxisEnd-FYAxisStart);
         { Execute the plot loop }
         for C:=1 to FCounterResolution do
         begin
            { calculate the segment color }
            cl:=0;
            if RIndex>-1 then cl:=cl+round(Equations[RIndex].Compute);
            if GIndex>-1 then cl:=cl+round(Equations[GIndex].Compute) shl 8;
            if BIndex>-1 then cl:=cl+round(Equations[BIndex].Compute) shl 16;
            pen.color:=cl;
            { calculate the screen coordinates }
            X:=Equations[Xindex].Compute;
            Y:=Equations[Yindex].Compute;
            Cx:=Round( (x-FXAxisStart)*XScale );
            Cy:=Round( (FYAxisEnd-y)*YScale );
            { plot with lines }
            if (c=1) or (cl=0) then MoveTo(Cx,Cy) else LineTo(Cx,Cy);
            { calculate the next value for the Counter variable }
            FCounterVar:=FCounterVar+Step;
         end;
      end;
end;

procedure TForm1.btGraphicClick(Sender: TObject);
begin
   atEquationMemo1.Refresh; 
   UpdateMathModel; 
   PaintGraphic;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
   fmInfo.ShowModal;
end;

procedure TForm1.Simplifiedmodel1Click(Sender: TObject);
begin
   fmSimplifiedModel.MathEvaluator:=atMathEvaluator1;
   fmSimplifiedModel.Show;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
   FMathModel.Free;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
   with OpenDialog1 do
      if Execute then
         LoadModelFromFile( FileName );
end;

procedure TForm1.Save1Click(Sender: TObject);
begin
   with FMathModel do
      if FileName>'' then
         SaveModelToFile( FileName )
      else
         Saveas1Click( Sender );
end;

procedure TForm1.Saveas1Click(Sender: TObject);
begin
   with SaveDialog1 do
      if Execute then
         SaveModelToFile( FileName );
end;

procedure TForm1.LoadModelFromFile( AFileName:string );
begin
   if FileExists(AFileName) then
   begin
      FMathModel.FileName:=AFileName;
      ReadComponentResFile( AFileName, FMathModel );
      UpdateScreen;
      { plot refresh }
      btGraphicClick(self);
   end;
end;

procedure TForm1.SaveModelToFile( AFileName:string );
begin
   if (FMathModel.FileName=AFileName) or not FileExists(AFileName) or
      (MessageDlg(Format('File "%s" already exists. Overwrite?',[AFileName]),mtConfirmation,[mbYes,mbNo],0)=mrYes) then
   begin
      UpdateMathModel;
      FMathModel.FileName:=AFileName;
      WriteComponentResFile( AFileName, FMathModel );
   end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
   FMathModel.FileName:='';
   LoadModelFromFile( 'Default.mat' );
end;

procedure TForm1.New1Click(Sender: TObject);
begin
   atEquationMemo1.Equations.Clear;
   FMathModel.FileName:='';
end;

procedure TForm1.Quit1Click(Sender: TObject);
begin
   Application.Terminate;
end;

procedure TForm1.cbSyntaxClick(Sender: TObject);
begin
   atEquationMemo1.PlainText:=not cbSyntax.checked;
   atEquationMemo1.Refresh;
end;

end.
